@@ -17,6 +17,7 @@
 #include "feedback-manager.h"
 #include "app-grid.h"
 #include "app-grid-button.h"
+#include "app-grid-folder-button.h"
 #include "app-list-model.h"
 #include "favorite-list-model.h"
 #include "shell.h"
@@ -25,6 +26,8 @@
 #include "gtk-list-models/gtksortlistmodel.h"
 #include "gtk-list-models/gtkfilterlistmodel.h"
 
+#include <gmobile.h>
+
 enum {
   PROP_0,
   PROP_FILTER_ADAPTIVE,
@@ -42,6 +45,7 @@ typedef struct _PhoshAppGridPrivate PhoshAppGridPrivate;
 struct _PhoshAppGridPrivate {
   GtkFilterListModel *model;
 
+  GtkWidget *deck;
   GtkWidget *search;
   GtkWidget *apps;
   GtkWidget *favs;
@@ -50,6 +54,17 @@ struct _PhoshAppGridPrivate {
   GtkWidget *btn_adaptive;
   GtkWidget *btn_adaptive_img;
   GtkWidget *btn_adaptive_lbl;
+  GtkWidget *empty_folder_label;
+  GtkWidget *folder_stack;
+  GtkWidget *folder_name_btn;
+  GtkWidget *folder_name_img;
+  GtkWidget *folder_name_entry;
+  GtkWidget *folder_name_label;
+  GtkWidget *folder_apps;
+
+  PhoshFolderInfo *open_folder;
+  int              open_folder_idx;
+  GListModel      *folder_model;
 
   char *search_string;
   gboolean filter_adaptive;
@@ -101,6 +116,58 @@ phosh_app_grid_get_property (GObject    *object,
 }
 
 
+static void
+show_main_grid (PhoshAppGrid *self)
+{
+  PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
+  GtkFlowBoxChild *button = NULL;
+  hdy_deck_set_visible_child_name (HDY_DECK (priv->deck), "main_grid");
+
+  if (priv->open_folder == NULL)
+    return;
+
+  g_signal_handlers_disconnect_by_data (priv->folder_model, self);
+  priv->folder_model = NULL;
+  g_clear_object (&priv->open_folder);
+
+  if (priv->open_folder_idx == -1)
+    return;
+
+  /* Focus the first valid button from current index to 0 */
+  for (int idx = priv->open_folder_idx; idx >= 0; idx--) {
+    button = gtk_flow_box_get_child_at_index (GTK_FLOW_BOX (priv->apps), idx);
+    if (button != NULL)
+      break;
+  }
+
+  gtk_widget_grab_focus (GTK_WIDGET (button));
+}
+
+
+static void
+show_folder_page (PhoshAppGrid *self)
+{
+  PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
+  const char *page_name;
+  GtkFlowBoxChild *button = NULL;
+
+  if (g_app_info_should_show (G_APP_INFO (priv->open_folder))) {
+    page_name = "folder_grid";
+    button = gtk_flow_box_get_child_at_index (GTK_FLOW_BOX (priv->folder_apps), 0);
+  } else {
+    g_autofree char *label;
+    page_name = "empty_folder";
+    label = g_strdup_printf ("%s folder is empty", phosh_folder_info_get_name (priv->open_folder));
+    gtk_label_set_label (GTK_LABEL (priv->empty_folder_label), label);
+  }
+
+  gtk_stack_set_visible_child_name (GTK_STACK (priv->folder_stack), page_name);
+
+  if (button != NULL)
+    gtk_widget_grab_focus (GTK_WIDGET (button));
+}
+
+
 static void
 app_launched_cb (GtkWidget    *widget,
                  GAppInfo     *info,
@@ -111,6 +178,64 @@ app_launched_cb (GtkWidget    *widget,
 }
 
 
+static GtkWidget *
+create_folder_app_launcher (gpointer item, gpointer self)
+{
+  GtkWidget *btn;
+  PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
+
+  btn = phosh_app_grid_button_new (G_APP_INFO (item));
+  phosh_app_grid_button_set_folder_info (PHOSH_APP_GRID_BUTTON (btn), priv->open_folder);
+  g_signal_connect (btn, "app-launched", G_CALLBACK (app_launched_cb), self);
+
+  gtk_widget_show (btn);
+
+  return btn;
+}
+
+
+static int
+get_app_info_index (PhoshAppGrid *self, GAppInfo *app_info)
+{
+  PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
+
+  for (int idx = 0;; idx++) {
+    g_autoptr (GAppInfo) info = g_list_model_get_item (G_LIST_MODEL (priv->model), idx);
+
+    if (info == NULL)
+      return -1;
+
+    if (g_app_info_equal (info, app_info))
+      return idx;
+  }
+
+  return -1;
+}
+
+
+static void
+folder_launched_cb (GtkWidget       *widget,
+                    PhoshFolderInfo *info,
+                    PhoshAppGrid    *self)
+{
+  PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
+  GListModel *model = phosh_folder_info_get_app_infos (info);
+
+  hdy_deck_set_visible_child_name (HDY_DECK (priv->deck), "folder_page");
+  g_object_bind_property (info, "name", priv->folder_name_label, "label", G_BINDING_SYNC_CREATE);
+  priv->folder_model = model;
+  g_set_object (&priv->open_folder, info);
+  priv->open_folder_idx = get_app_info_index (self, G_APP_INFO (info));
+  g_signal_connect_object (model, "items-changed", G_CALLBACK (show_folder_page), self, G_CONNECT_SWAPPED);
+  gtk_entry_set_text (GTK_ENTRY (priv->folder_name_entry), "");
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->folder_name_btn), FALSE);
+  show_folder_page (self);
+
+  gtk_flow_box_bind_model (GTK_FLOW_BOX (priv->folder_apps),
+                           model, create_folder_app_launcher, self, NULL);
+}
+
+
 static int
 sort_apps (gconstpointer a,
            gconstpointer b,
@@ -204,20 +329,6 @@ filter_adaptive (PhoshAppGrid *self, GDesktopAppInfo *info)
 }
 
 
-static const char *(*app_attr[]) (GAppInfo *info) = {
-  g_app_info_get_display_name,
-  g_app_info_get_name,
-  g_app_info_get_description,
-  g_app_info_get_executable,
-};
-
-
-static const char *(*desktop_attr[]) (GDesktopAppInfo *info) = {
-  g_desktop_app_info_get_generic_name,
-  g_desktop_app_info_get_categories,
-};
-
-
 static gboolean
 search_apps (gpointer item, gpointer data)
 {
@@ -225,7 +336,6 @@ search_apps (gpointer item, gpointer data)
   PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
   GAppInfo *info = item;
   const char *search = NULL;
-  const char *str = NULL;
 
   g_return_val_if_fail (priv != NULL, TRUE);
   g_return_val_if_fail (priv->search != NULL, TRUE);
@@ -238,59 +348,19 @@ search_apps (gpointer item, gpointer data)
   }
 
   /* filter out favorites when not searching */
-  if (search == NULL || strlen (search) == 0) {
+  if (gm_str_is_null_or_empty (search)) {
+    if (PHOSH_IS_FOLDER_INFO (info))
+      return phosh_folder_info_refilter (PHOSH_FOLDER_INFO (info), search);
     if (phosh_favorite_list_model_app_is_favorite (NULL, info))
       return FALSE;
 
     return TRUE;
   }
 
-  for (int i = 0; i < G_N_ELEMENTS (app_attr); i++) {
-    g_autofree char *folded = NULL;
+  if (PHOSH_IS_FOLDER_INFO (info))
+    return phosh_folder_info_refilter (PHOSH_FOLDER_INFO (info), search);
 
-    str = app_attr[i] (info);
-
-    if (!str || *str == '\0')
-      continue;
-
-    folded = g_utf8_casefold (str, -1);
-
-    if (strstr (folded, search))
-      return TRUE;
-  }
-
-  if (G_IS_DESKTOP_APP_INFO (info)) {
-    const char * const *kwds;
-
-    for (int i = 0; i < G_N_ELEMENTS (desktop_attr); i++) {
-      g_autofree char *folded = NULL;
-
-      str = desktop_attr[i] (G_DESKTOP_APP_INFO (info));
-
-      if (!str || *str == '\0')
-        continue;
-
-      folded = g_utf8_casefold (str, -1);
-
-      if (strstr (folded, search))
-        return TRUE;
-    }
-
-    kwds = g_desktop_app_info_get_keywords (G_DESKTOP_APP_INFO (info));
-
-    if (kwds) {
-      int i = 0;
-
-      while ((str = kwds[i])) {
-        g_autofree char *folded = g_utf8_casefold (str, -1);
-        if (strstr (folded, search))
-          return TRUE;
-        i++;
-      }
-    }
-  }
-
-  return FALSE;
+  return phosh_util_matches_app_info (info, search);
 }
 
 
@@ -309,14 +379,14 @@ create_favorite_launcher (gpointer item,
 }
 
 static void
-toggle_favorties_revealer (PhoshAppGrid *self)
+toggle_favorites_revealer (PhoshAppGrid *self)
 {
   PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
   PhoshFavoriteListModel *favorites = phosh_favorite_list_model_get_default ();
 
   /* Hide favorites when there are none or a search is in progress */
   gboolean criteria = (g_list_model_get_n_items (G_LIST_MODEL (favorites)) == 0) ||
-    !STR_IS_NULL_OR_EMPTY (priv->search_string);
+    !gm_str_is_null_or_empty (priv->search_string);
 
   gtk_revealer_set_reveal_child (GTK_REVEALER (priv->favs_revealer), !criteria);
 }
@@ -331,7 +401,7 @@ favorites_changed (GListModel   *list,
 {
   PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
 
-  toggle_favorties_revealer (self);
+  toggle_favorites_revealer (self);
 
   /* We don't show favorites in the main list, filter them out */
   gtk_filter_list_model_refilter (priv->model);
@@ -342,10 +412,17 @@ static GtkWidget *
 create_launcher (gpointer item,
                  gpointer self)
 {
-  GtkWidget *btn = phosh_app_grid_button_new (G_APP_INFO (item));
+  GtkWidget *btn;
 
-  g_signal_connect (btn, "app-launched",
-                    G_CALLBACK (app_launched_cb), self);
+  if (PHOSH_IS_FOLDER_INFO (item)) {
+    btn = phosh_app_grid_folder_button_new_from_folder_info (item);
+    g_signal_connect (btn, "folder-launched",
+                      G_CALLBACK (folder_launched_cb), self);
+  } else {
+    btn = phosh_app_grid_button_new (G_APP_INFO (item));
+    g_signal_connect (btn, "app-launched",
+                      G_CALLBACK (app_launched_cb), self);
+  }
 
   gtk_widget_show (btn);
 
@@ -401,6 +478,8 @@ phosh_app_grid_init (PhoshAppGrid *self)
                                   G_ACTION_GROUP (priv->actions));
   action = (GAction*) g_property_action_new ("filter-adaptive", self, "filter-adaptive");
   g_action_map_add_action (G_ACTION_MAP (priv->actions), action);
+
+  toggle_favorites_revealer (self);
 }
 
 
@@ -410,6 +489,7 @@ phosh_app_grid_dispose (GObject *object)
   PhoshAppGrid *self = PHOSH_APP_GRID (object);
   PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
 
+  g_clear_object (&priv->open_folder);
   g_clear_object (&priv->actions);
   g_clear_object (&priv->model);
   g_clear_object (&priv->settings);
@@ -439,6 +519,10 @@ phosh_app_grid_key_press_event (GtkWidget   *widget,
   PhoshAppGrid *self = PHOSH_APP_GRID (widget);
   PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
 
+  /* Don't search when folder is open */
+  if (priv->open_folder != NULL)
+    return GDK_EVENT_PROPAGATE;
+
   return gtk_search_entry_handle_event (GTK_SEARCH_ENTRY (priv->search),
                                         (GdkEvent *) event);
 }
@@ -460,7 +544,7 @@ do_search (PhoshAppGrid *self)
                                     ACTIVE_SEARCH_CLASS);
   }
 
-  toggle_favorties_revealer (self);
+  toggle_favorites_revealer (self);
   gtk_filter_list_model_refilter (priv->model);
 
   priv->debounce = 0;
@@ -468,6 +552,39 @@ do_search (PhoshAppGrid *self)
 }
 
 
+static void
+on_folder_edit_toggled (PhoshAppGrid *self, GtkToggleButton *toggle_btn)
+{
+  PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
+  gboolean active = gtk_toggle_button_get_active (toggle_btn);
+
+  if (active) {
+    const char *folder_name = phosh_folder_info_get_name (priv->open_folder);
+    gtk_entry_set_text (GTK_ENTRY (priv->folder_name_entry), folder_name);
+    gtk_widget_grab_focus (priv->folder_name_entry);
+    gtk_image_set_from_icon_name (GTK_IMAGE (priv->folder_name_img),
+                                  "emblem-ok-symbolic", GTK_ICON_SIZE_BUTTON);
+  } else {
+    const char *folder_name = gtk_entry_get_text (GTK_ENTRY (priv->folder_name_entry));
+    if (gm_str_is_null_or_empty (folder_name))
+      return;
+    phosh_folder_info_set_name (priv->open_folder, folder_name);
+    gtk_entry_set_text (GTK_ENTRY (priv->folder_name_entry), "");
+    gtk_image_set_from_icon_name (GTK_IMAGE (priv->folder_name_img),
+                                  "document-edit-symbolic", GTK_ICON_SIZE_BUTTON);
+  }
+}
+
+
+static void
+on_folder_entry_activated (PhoshAppGrid *self, GtkEntry *entry)
+{
+  PhoshAppGridPrivate *priv = phosh_app_grid_get_instance_private (self);
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->folder_name_btn), FALSE);
+}
+
+
 static void
 search_changed (GtkSearchEntry *entry,
                 PhoshAppGrid   *self)
@@ -607,16 +724,27 @@ phosh_app_grid_class_init (PhoshAppGridClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, btn_adaptive);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, btn_adaptive_img);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, btn_adaptive_lbl);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, deck);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, empty_folder_label);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, favs);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, favs_revealer);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, folder_apps);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, folder_name_btn);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, folder_name_img);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, folder_name_entry);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, folder_name_label);
+  gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, folder_stack);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, scrolled_window);
   gtk_widget_class_bind_template_child_private (widget_class, PhoshAppGrid, search);
 
+  gtk_widget_class_bind_template_callback (widget_class, on_folder_edit_toggled);
+  gtk_widget_class_bind_template_callback (widget_class, on_folder_entry_activated);
   gtk_widget_class_bind_template_callback (widget_class, search_changed);
   gtk_widget_class_bind_template_callback (widget_class, search_preedit_changed);
   gtk_widget_class_bind_template_callback (widget_class, search_activated);
   gtk_widget_class_bind_template_callback (widget_class, search_gained_focus);
   gtk_widget_class_bind_template_callback (widget_class, search_lost_focus);
+  gtk_widget_class_bind_template_callback (widget_class, show_main_grid);
 
   signals[APP_LAUNCHED] = g_signal_new ("app-launched",
                                         G_TYPE_FROM_CLASS (klass),
@@ -650,6 +778,14 @@ phosh_app_grid_reset (PhoshAppGrid *self)
   gtk_adjustment_set_value (adjustment, 0);
   gtk_entry_set_text (GTK_ENTRY (priv->search), "");
   g_clear_pointer (&priv->search_string, g_free);
+
+  /* Do not focus last folder */
+  priv->open_folder_idx = -1;
+  /* Set duration to 0 to avoid the simultaneous animation of grid sliding up and deck sliding
+   * right. If not done, it feels like the deck is spiraling diagonally up. */
+  hdy_deck_set_transition_duration (HDY_DECK (priv->deck), 0);
+  show_main_grid (self);
+  hdy_deck_set_transition_duration (HDY_DECK (priv->deck), 200);
 }
 
 
@@ -672,6 +808,11 @@ phosh_app_grid_handle_search (PhoshAppGrid *self, GdkEvent *event)
 
   g_return_val_if_fail (PHOSH_IS_APP_GRID (self), GDK_EVENT_PROPAGATE);
   priv = phosh_app_grid_get_instance_private (self);
+
+  /* Prevent stealing of focus when folder is open and it's name entry is active */
+  if (priv->open_folder != NULL)
+    return GDK_EVENT_PROPAGATE;
+
   ret = gtk_search_entry_handle_event (GTK_SEARCH_ENTRY (priv->search), event);
   if (ret == GDK_EVENT_STOP)
     gtk_entry_grab_focus_without_selecting (GTK_ENTRY (priv->search));
