@@ -60,6 +60,19 @@ on_power_off_activated (GSimpleAction *action, GVariant *param, gpointer data)
 }
 
 
+static void
+on_suspend_activated (GSimpleAction *action, GVariant *param, gpointer data)
+{
+  PhoshPowerMenuManager *self = PHOSH_POWER_MENU_MANAGER (data);
+
+  g_return_if_fail (PHOSH_IS_POWER_MENU_MANAGER (self));
+
+  close_menu (self);
+  g_action_group_activate_action (G_ACTION_GROUP (phosh_shell_get_default ()),
+                                  "suspend.trigger-suspend", NULL);
+}
+
+
 static void
 on_screen_lock_activated (GSimpleAction *action, GVariant *param, gpointer data)
 {
@@ -106,6 +119,8 @@ static void
 on_power_menu_activated (GSimpleAction *action, GVariant *param, gpointer data)
 {
   PhoshPowerMenuManager *self = PHOSH_POWER_MENU_MANAGER (data);
+  gboolean locked, show_suspend;
+  GAction *suspend_action;
 
   if (self->dialog) {
     on_power_menu_done (self);
@@ -118,29 +133,32 @@ on_power_menu_activated (GSimpleAction *action, GVariant *param, gpointer data)
   g_signal_connect_swapped (self->dialog, "done",
                             G_CALLBACK (on_power_menu_done), self);
 
+  /* Show suspend on lock screen when available */
+  locked = phosh_shell_get_locked (phosh_shell_get_default ());
+  suspend_action = g_action_map_lookup_action (G_ACTION_MAP (self->menu_actions), "suspend");
+  show_suspend = locked && g_action_get_enabled (suspend_action);
+  phosh_power_menu_set_show_suspend (self->dialog, show_suspend);
+
   gtk_widget_show (GTK_WIDGET (self->dialog));
 }
 
 
 static void
-on_shell_state_changed (PhoshPowerMenuManager *self, GParamSpec *pspec, PhoshShell *shell)
+on_shell_locked_changed (PhoshPowerMenuManager *self, GParamSpec *pspec, PhoshShell *shell)
 {
   GAction *action;
-  gboolean enable = TRUE;
-  PhoshShellStateFlags state;
+  gboolean locked;
 
   g_return_if_fail (PHOSH_IS_POWER_MENU_MANAGER (self));
   g_return_if_fail (PHOSH_IS_SHELL (shell));
 
-  state = phosh_shell_get_state (shell);
-  if (state & PHOSH_STATE_LOCKED)
-    enable = FALSE;
+  locked = phosh_shell_get_locked (shell);
 
   /* TODO: Make them work on the lock screen too */
   action = g_action_map_lookup_action (G_ACTION_MAP (self->menu_actions), "screen-lock");
-  g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enable);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (action), !locked);
   action = g_action_map_lookup_action (G_ACTION_MAP (self->menu_actions), "poweroff");
-  g_simple_action_set_enabled (G_SIMPLE_ACTION (action), enable);
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (action), !locked);
 }
 
 
@@ -168,10 +186,11 @@ phosh_power_menu_manager_class_init (PhoshPowerMenuManagerClass *klass)
 
 
 static GActionEntry menu_entries[] = {
+  { .name = "emergency-call", .activate = on_emergency_call_activated },
   { .name = "poweroff", .activate = on_power_off_activated },
   { .name = "screen-lock", .activate = on_screen_lock_activated },
   { .name = "screenshot", .activate = on_screenshot_activated },
-  { .name = "emergency-call", .activate = on_emergency_call_activated },
+  { .name = "suspend", .activate = on_suspend_activated },
 };
 
 
@@ -184,6 +203,7 @@ static void
 phosh_power_menu_manager_init (PhoshPowerMenuManager *self)
 {
   GAction *src_action, *dst_action;
+  g_autoptr (GSettings) phosh_settings = g_settings_new ("sm.puri.phosh");
 
   g_action_map_add_action_entries (G_ACTION_MAP (phosh_shell_get_default ()),
                                    entries,
@@ -201,12 +221,18 @@ phosh_power_menu_manager_init (PhoshPowerMenuManager *self)
   dst_action = g_action_map_lookup_action (G_ACTION_MAP (self->menu_actions), "emergency-call");
   g_object_bind_property (src_action, "enabled", dst_action, "enabled", G_BINDING_SYNC_CREATE);
 
+  g_settings_bind (phosh_settings,
+                   "enable-suspend",
+                   g_action_map_lookup_action (G_ACTION_MAP (self->menu_actions), "suspend"),
+                   "enabled",
+                   G_SETTINGS_BIND_GET);
+
   g_signal_connect_object (phosh_shell_get_default (),
-                           "notify::shell-state",
-                           G_CALLBACK (on_shell_state_changed),
+                           "notify::locked",
+                           G_CALLBACK (on_shell_locked_changed),
                            self,
                            G_CONNECT_SWAPPED);
-  on_shell_state_changed (self, NULL, phosh_shell_get_default ());
+  on_shell_locked_changed (self, NULL, phosh_shell_get_default ());
 }
 
 
