@@ -6,7 +6,7 @@
 
 #define G_LOG_DOMAIN "phosh-audio_device-row"
 
-#include "config.h"
+#include "phosh-config.h"
 
 #include "audio-device.h"
 #include "audio-device-row.h"
@@ -165,7 +165,14 @@ phosh_audio_device_row_new (PhoshAudioDevice *audio_device)
                        NULL);
 }
 
-
+/**
+ * phosh_audio_device_row_get_audio_device:
+ * @self: An audio device row
+ *
+ * Get the audio device associated with this row
+ *
+ * Returns:(transfer none): The audio device
+ */
 PhoshAudioDevice *
 phosh_audio_device_row_get_audio_device (PhoshAudioDeviceRow *self)
 {
