@@ -10,8 +10,6 @@
 #include <gtk/gtk.h>
 #include <gio/gdesktopappinfo.h>
 
-#define STR_IS_NULL_OR_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
-
 #define phosh_async_error_warn(err, ...) \
   phosh_error_warnv (G_LOG_DOMAIN, err, G_IO_ERROR, G_IO_ERROR_CANCELLED, __VA_ARGS__)
 
@@ -31,7 +29,6 @@ gboolean         phosh_error_warnv (const char  *log_domain,
                                     ...) G_GNUC_PRINTF(5, 6);
 int              phosh_create_shm_file (off_t size);
 char            *phosh_util_escape_markup (const char *markup, gboolean allow_markup);
-char            *phosh_util_local_date (void);
 gboolean         phosh_util_gesture_is_touch (GtkGestureSingle *gesture);
 gboolean         phosh_util_have_gnome_software (gboolean scan);
 void             phosh_util_toggle_style_class (GtkWidget *widget, const char *style_class, gboolean toggle);
@@ -39,4 +36,7 @@ const char      *phosh_util_get_stylesheet (const char *theme_name);
 gboolean         phosh_clear_fd (int *fd, GError **err);
 const char      *phosh_util_get_icon_by_wifi_strength (guint strength, gboolean is_connecting);
 gboolean         phosh_util_file_equal (GFile *file1, GFile *file2);
-GdkPixbuf       *phosh_util_data_uri_to_pixbuf (const char *url, GError **error);
+GdkPixbuf       *phosh_util_data_uri_to_pixbuf (const char *uri, GError **error);
+gboolean         phosh_util_matches_app_info (GAppInfo *info, const char *search);
+GStrv            phosh_util_append_to_strv (GStrv array, const char *element);
+GStrv            phosh_util_remove_from_strv (GStrv array, const char *element);
