@@ -46,9 +46,9 @@ static void
 test_phosh_util_data_uri_to_pixbuf (void)
 {
   /* smallest valid png. see https://evanhahn.com/worlds-smallest-png/ */
-  const char *valid_uri = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQAAAAA3bvkkAAAACklEQVR4AWNgAAAAAgABc3UBGAAAAABJRU5ErkJggg==";
-  /* not a png */
-  const char *invalid_uri = "data:image/png;base64,aGkgdGhlcmUh";
+  const char *valid_uri = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABAQAAAAA3bvkkAAAAC"
+    "klEQVR4AWNgAAAAAgABc3UBGAAAAABJRU5ErkJggg==";
+  const char *not_a_png = "data:image/png;base64,aGkgdGhlcmUh";
   GdkPixbuf* pixbuf = NULL;
   g_autoptr (GError) error = NULL;
 
@@ -57,9 +57,14 @@ test_phosh_util_data_uri_to_pixbuf (void)
   g_assert_nonnull (pixbuf);
   g_assert_finalize_object (pixbuf);
 
-  pixbuf = phosh_util_data_uri_to_pixbuf (invalid_uri, &error);
+  pixbuf = phosh_util_data_uri_to_pixbuf (not_a_png, &error);
   g_assert_null (pixbuf);
   g_assert_error (error, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_UNKNOWN_TYPE);
+
+  g_clear_error (&error);
+  pixbuf = phosh_util_data_uri_to_pixbuf ("not-an_uri", &error);
+  g_assert_null (pixbuf);
+  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_FAILED);
 }
 
 
